/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.security.InvalidParameterException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

/**
 * Controls the execution of many tasks. The tasks can be finite
 * or infinite. It is not possible to mix finite with infinite tasks.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class TaskController {

    // Controls the type of tasks being added. 
    private static final int TASK_TYPE_YET_DEFINED = 0;
    private static final int FINITE_TASK_TYPE = 1;
    private static final int INFINITE_TASK_TYPE = 2;
    private int currentTaskType = TASK_TYPE_YET_DEFINED;

    // The main form that holds the tasks monitoring components.
    private JFrame mainFrame = null;

    // Store the super component. The monitoring frame will be
    // positioned according to this component.
    private Component superComponent = null;

    // Information of each task. The position of these information 
    // is important. The progress bar at a position X is the one
    // being used to monitor task at position X.
    private ArrayList tasks = new ArrayList();
    private ArrayList progressBars = new ArrayList();
    private ArrayList stopButtons = new ArrayList();
    private ArrayList messages = new ArrayList();
    private ArrayList results = new ArrayList();

    // Delay to inform the progress.
    private final int QUARTER_OF_SECOND = 250;
    private javax.swing.Timer timer = null;

    /**
     * Default constructor. 
     */
    public TaskController() {

        super();
        mainFrame = new JFrame();
    }

    /**
     * Constructor that can receive a name. This name
     * will be placed on the title of the frame.
     * @param controllerName The name.
     */
    public TaskController(String controllerName) {

        super();
        mainFrame = new JFrame(controllerName);
    }

    /**
     * @param controllerName
     * @param superComponent
     */
    /**
     * Constructor that can receive a name and the super component. This name
     * will be placed on the title of the frame. And the frame will be positioned
     * according to the super component.
     * @param controllerName The name.
     * @param superComponent The component that holds the progress monitoring frame.
     */
    public TaskController(String controllerName, Component superComponent) {

        super();
        mainFrame = new JFrame(controllerName);
        this.superComponent = superComponent;
    }

    /**
     * Provides the textual representation of a total time in seconds.
     * @param time The time in seconds.
     * @return The textual representation.
     */
    private String getTimeText(long time) {

        // If less than a minute
        if (time < 60) {
            return time + "  " + InternalResource.get(InternalResource.SECONDS);
        }

        long minutes = time / 60;
        long seconds = time - (minutes * 60);

        // Seconds are presented in two characters all the time.
        String addZero = seconds < 10 ? "0" : "";

        // Return the representation in minutes.
        return minutes + ":" + addZero + seconds + " "
            + InternalResource.get(InternalResource.MINUTES);

    }

    /**
     * Return a textual representation of a progress of an finite task.
     * @param currentStep Indiate the current step.
     * @param steps Indicate the total numbe of current steps.
     * @param elapsed The total of seconds that the task has being executed.
     * @param remaining the total of seconds to finish the task.
     * @return The textual representation.
     */
    private String getFiniteTaskMessage(int currentStep, int steps,
        long elapsed, long remaining) {

        return "(" + currentStep + " "
            + InternalResource.get(InternalResource.STEPS_SEPARATION) + " "
            + steps + ") " + InternalResource

            .get(InternalResource.TIME_ELAPSED) + " " + getTimeText(elapsed)
            + " - " + InternalResource.get(InternalResource.TIME_REMAINING)
            + " " + getTimeText(remaining);
    }

    /**
     * Return a textual representation of a progress of an infinite task.
     * @param progress The number representing the progress.
     * @return The textual representation.
     */
    private String getInfiniteTaskMessage(long progress) {

        return InternalResource.get(InternalResource.INDICATED_PROGRESS) + " "
            + progress + " " + InternalResource.get(InternalResource.UNITS);
    }

    /**
     * Inform that an exception has occurred in a task.
     */
    private void reportException(final Task task, final String message) {

        Runnable doSetProgressBarValue = new Runnable() {

            /* (non-Javadoc)
             * @see java.lang.Runnable#run()
             */
            public void run() {

                JOptionPane.showMessageDialog(mainFrame, message,
                    InternalResource.get(InternalResource.TASK_EXCEPTION_TITLE)
                        + "(" + task.getTaskName() + ")",
                    JOptionPane.ERROR_MESSAGE);
            }
        };

        // Show the dialog when possible.
        SwingUtilities.invokeLater(doSetProgressBarValue);
    }

    /**
     * Update the overal progress of all tasks.
     */
    class TimerListener implements ActionListener {

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent evt) {

            boolean hasRunningTask = false;
            for (int i = 0; i < tasks.size(); i++) {

                // Retrieve the information of the current task.
                Task task = (Task)tasks.get(i);
                JLabel message = (JLabel)messages.get(i);
                JProgressBar progressBar = null;

                // Only finite tasks have progress bar.
                if (currentTaskType == FINITE_TASK_TYPE) {
                    progressBar = (JProgressBar)progressBars.get(i);
                }

                // If the task is not running.
                if (task.isFinished() || task.isInterrupted()) {

                    // Disable the stop button.
                    JButton stopButton = (JButton)stopButtons.get(i);
                    stopButton.setEnabled(false);

                    // Only finite classes can finish their task.
                    if (currentTaskType == FINITE_TASK_TYPE) {

                        // Update the current status.
                        updateStatus(progressBar, task.isFinished() ? 100 : 0);

                        // The task could be interrupted.
                        if (task.isInterrupted()) {

                            // If the task has been interrupted by the user.
                            if (!task.isInterruptedByException()) {
                                message
                                        .setText(InternalResource
                                                .get(InternalResource.TASK_INTERRUPTED_SHORT_MESSAGE));
                            }

                            // Verify if the task has been interrupted by an exception.
                            Throwable exception = task.getTaskException();
                            if (exception != null) {

                                // Show the exception.
                                reportException(task, ExceptionSupport
                                        .throwableToString(exception));

                                message.setText(InternalResource
                                        .get(InternalResource.TASK_EXCEPTION));

                                // Stop the task.
                                task.stopTask();
                            }

                        } else {

                            // The task has ended normally than just update the progress.
                            message.setText(InternalResource
                                    .get(InternalResource.TASK_FINISHED)
                                + " "
                                + InternalResource
                                        .get(InternalResource.TIME_ELAPSED)
                                + getTimeText(task.getElapsedTime()));
                        }
                    }

                } else {

                    // There is at least one running task.
                    hasRunningTask = true;

                    // Update the progress bar only if the task is finite
                    if (currentTaskType == FINITE_TASK_TYPE) {
                        updateStatus(progressBar, task
                                .getCurrentProgressNormalized());
                    }

                    // Update the message according to the type of task.
                    if (currentTaskType == FINITE_TASK_TYPE) {
                        message.setText(getFiniteTaskMessage(task
                                .getCurrentStep(), task.getSteps(), task
                                .getElapsedTime(), task.getRemainingTime()));
                    } else {
                        message.setText(getInfiniteTaskMessage(task
                                .getCurrentProgress()));
                    }
                }
            }

            // Stop the timer if there is no task running.
            if (!hasRunningTask) {
                timer.stop();
            }
        }
    }

    /**
     * Waits until all tasks have been finished and return all the results.
     * This method should be called only when the type of tasks are finite.
     * @param sleepTime The amount of miliseconds to wait during each verification.
     * @return All the results generated by the tasks.
     */
    public Object[] getResults(long sleepTime) {

        if (currentTaskType != FINITE_TASK_TYPE) {
            throw new InvalidParameterException(InternalResource
                    .get(InternalResource.RESULT_EXCEPTION));
        }

        while (hasResultsToBeRetrieved()) {

            Object result = getResult();

            if (result != null) {
                results.add(result);
            }

            try {
                Thread.sleep(sleepTime);
            }
            catch(InterruptedException e) {
                // Silence
            }
        }

        return results.toArray();
    }

    /**
     * Confirm if the user really wants to interrupt a specific task.
     * Return true if the user confirm the dialog
     */
    private boolean confirmTaskInterrupt(final String taskName) {

        boolean confirmInterrupt;

        Object[] options = {InternalResource.get(InternalResource.CONFIRM),
            InternalResource.get(InternalResource.CANCEL)};

        int n = JOptionPane.showOptionDialog(null, InternalResource
                .get(InternalResource.TASK_INTERRUPTION_CONFIRMATION)
            + " (" + taskName + ")", InternalResource
                .get(InternalResource.TASK_INTERRUPTION_CONFIRMATION_TITLE),
            JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null,
            options, InternalResource.get(InternalResource.CANCEL));

        confirmInterrupt = (n == JOptionPane.YES_OPTION);

        return confirmInterrupt;
    }

    /**
     * Confirm if the user really wants to interrupt all tasks.
     * Return true if the user confirm the dialog
     */
    private boolean confirmExit() {

        boolean confirmExit;

        Object[] options = {InternalResource.get(InternalResource.CONFIRM),
            InternalResource.get(InternalResource.CANCEL)};

        int n = JOptionPane.showOptionDialog(null, InternalResource
                .get(InternalResource.INTERRUPT_ALL_TASKS), InternalResource
                .get(InternalResource.WINDOW_CLOSE_CONFIRMATION_TITLE),
            JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null,
            options, InternalResource.get(InternalResource.CANCEL));

        confirmExit = (n == JOptionPane.YES_OPTION);

        return confirmExit;
    }

    /**
     * Listen to any stop button of any task.
     *  
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class ButtonListener implements ActionListener {

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent evt) {

            // Retrieve the task based on the button.
            for (int i = 0; i < stopButtons.size(); i++) {

                JButton stopButton = (JButton)stopButtons.get(i);

                if (evt.getSource().equals(stopButton)) {

                    Task task = (Task)tasks.get(i);

                    if (confirmTaskInterrupt(task.getTaskName())) {

                        task.stopTask();

                        stopButton.setEnabled(false);
                    }
                }
            }
        }
    }

    /**
     * Add a finite task.
     * @param task The task.
     */
    public void addFiniteTask(Task task) {

        if (currentTaskType == TASK_TYPE_YET_DEFINED)
            currentTaskType = FINITE_TASK_TYPE;

        if (currentTaskType == INFINITE_TASK_TYPE)
            throw new InvalidParameterException(InternalResource
                    .get(InternalResource.TASK_FINITE_WITH_INFINITE_MESSAGE));

        tasks.add(task);
    }

    /**
     * Add a infinite task.
     * @param task The task.
     */
    public void addInfiniteTask(Task task) {

        if (currentTaskType == TASK_TYPE_YET_DEFINED)
            currentTaskType = INFINITE_TASK_TYPE;

        if (currentTaskType == FINITE_TASK_TYPE)
            throw new InvalidParameterException(InternalResource
                    .get(InternalResource.TASK_INFINITE_WITH_FINITE_MESSAGE));

        tasks.add(task);
    }

    /**
     * Verify if there is at least one task running.
     * @return True if there is at least one task running.
     */
    public boolean hasRunningTasks() {

        for (int i = 0; i < tasks.size(); i++) {

            Task task = (Task)tasks.get(i);

            if (!task.isFinished() && !task.isInterrupted()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Indicate if there is at least on return being generated by any task.
     * @return True if there is at least on return being generated by any task.
     */
    private boolean hasResultsToBeRetrieved() {

        boolean hasResultsToBeRetrieve = false;

        for (int i = 0; i < tasks.size(); i++) {

            Task task = (Task)tasks.get(i);

            // If the task is running than there is return being generated.
            if (!task.isFinished() && !task.isInterrupted()) {

                hasResultsToBeRetrieve = true;
                break;
            }

            // If there ir a task that has been finished and 
            // the result has not been retrieve yet than return true.
            if (task.isFinished()) {

                if (!task.isResultRestrieved()) {

                    hasResultsToBeRetrieve = true;
                }
            }
        }

        return hasResultsToBeRetrieve;
    }

    /**
     * Retrieve one result.
     * @return The result from a task.
     */
    private Object getResult() {

        for (int i = 0; i < tasks.size(); i++) {

            Task task = (Task)tasks.get(i);

            // If the task has been finished and the result 
            // has not been retrieved than return it.
            if (task.isFinished()) {

                if (!task.isResultRestrieved()) {

                    task.setResultRestrieved(true);

                    return task.getResult();
                }
            }
        }

        return null;
    }

    /**
     * Search for any intermediary result found.
     * @return An intermediary result.
     */
    public Object getIntermediaryResult() {

        for (int i = 0; i < tasks.size(); i++) {

            Task task = (Task)tasks.get(i);

            if (task.isIntermediaryResultAvailable()) {

                return task.getIntermediaryResult();
            }
        }

        return null;
    }

    /**
     * Update the progress bar with a specific percentage.
     * @param progressBar The progress bar to be updated.
     * @param i the percentage.
     */
    private void updateStatus(final JProgressBar progressBar, final int i) {

        Runnable doSetProgressBarValue = new Runnable() {

            /* (non-Javadoc)
             * @see java.lang.Runnable#run()
             */
            public void run() {

                progressBar.setValue(i);
            }
        };

        // Update whenever is possible.
        SwingUtilities.invokeLater(doSetProgressBarValue);
    }

    /**
     * Start all tasks and contruct the progress monitoring frame.
     */
    public void start() {

        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(0, 1));
        panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        // Add all tasks.
        for (int i = 0; i < tasks.size(); i++) {

            Task task = (Task)tasks.get(i);

            // All tasks have a execution panel
            JPanel executionPanel = new JPanel();

            // The execution panel has progress bars for finite task
            if (currentTaskType == FINITE_TASK_TYPE) {
                JProgressBar progressBar = new JProgressBar();
                progressBars.add(progressBar);
                executionPanel.add(progressBar);
            }

            // The execution panel has progress stop button.
            JButton interruptTaskButton = new JButton(InternalResource
                    .get(InternalResource.STOP));
            interruptTaskButton.addActionListener(new ButtonListener());
            stopButtons.add(interruptTaskButton);
            executionPanel.add(interruptTaskButton);

            // All tasks have a message panel.
            JPanel messagePanel = new JPanel();

            JLabel messageLabel = null;
            if (currentTaskType == FINITE_TASK_TYPE) {
                messageLabel = new JLabel(getFiniteTaskMessage(1, 1, 0, 0)
                    + "     ");
            } else {
                messageLabel = new JLabel(getInfiniteTaskMessage(0) + "     ");
            }
            messages.add(messageLabel);
            messagePanel.add(messageLabel);

            // Each task has a main specific panel that is 
            // composed by a execution and message panel.
            JPanel mainTaskPanel = new JPanel();
            mainTaskPanel.setLayout(new GridLayout(2, 1));
            mainTaskPanel.setBorder(BorderFactory.createTitledBorder(task
                    .getTaskName()));
            mainTaskPanel.add(executionPanel);
            mainTaskPanel.add(messagePanel);

            panel.add(mainTaskPanel);
        }

        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        mainFrame.setContentPane(new JScrollPane(panel));

        // Start all tasks
        for (int i = 0; i < tasks.size(); i++) {
            Task task = (Task)tasks.get(i);
            task.start();
        }

        // Control the user request to close the window.
        mainFrame.addWindowListener(new WindowAdapter() {

            /* (non-Javadoc)
             * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
             */
            public void windowClosing(WindowEvent e) {

                // Verify if all tasks have finished.
                boolean allTasksAreStoped = true;
                for (int i = 0; i < tasks.size(); i++) {
                    Task task = (Task)tasks.get(i);
                    if (!task.isFinished() && !task.isInterrupted()) {
                        allTasksAreStoped = false;
                    }
                }

                // If there is at least one running class than
                // request user confirmation.
                if (!allTasksAreStoped && !confirmExit()) {
                    return;
                }

                timer.stop();

                // Close all tasks.
                for (int i = 0; i < tasks.size(); i++) {
                    Task task = (Task)tasks.get(i);
                    task.stopTask();
                }

                tasks = new ArrayList();
                progressBars = new ArrayList();
                stopButtons = new ArrayList();
                messages = new ArrayList();

                mainFrame.setVisible(false);
                mainFrame.dispose();
            }
        });

        mainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

        mainFrame.pack();

        // Adjust the location of the frame if the super componet has been informed.
        if (superComponent != null) {

            // Get the middle position of the super component;
            int middleX = superComponent.getX()
                + (superComponent.getWidth() / 2);
            int middleY = superComponent.getY()
                + (superComponent.getHeight() / 2);

            // Get the progress information size.
            int frameX = middleX - mainFrame.getWidth() / 2;
            int frameY = middleY - mainFrame.getHeight() / 2;

            // Avoid getting outside the screen.
            if (frameY < 10)
                frameY = 10;

            Toolkit toolKit = Toolkit.getDefaultToolkit();

            Dimension screenSize = toolKit.getScreenSize();

            Dimension frameDimension = mainFrame.getSize();
            int frameHeight = (int)frameDimension.getHeight();

            // Avoid getting outside the screen.
            if (frameHeight > screenSize.getHeight()) {
                frameDimension.height = ((int)screenSize.getHeight()) - 20;
            }

            mainFrame.setLocation(frameX, frameY);
            mainFrame.setSize(frameDimension);

        } else {

            // Set the monitoring frame in the center of the screen.
            Toolkit toolKit = Toolkit.getDefaultToolkit();

            Dimension screenSize = toolKit.getScreenSize();

            // Get the middle position of the super component;
            int middleX = screenSize.width / 2;
            int middleY = screenSize.height / 2;

            // Get the progress information size.
            int frameX = middleX - mainFrame.getWidth() / 2;
            int frameY = middleY - mainFrame.getHeight() / 2;

            // Avoid getting outside the screen.
            if (frameY < 10)
                frameY = 10;

            Dimension frameDimension = mainFrame.getSize();
            int frameHeight = (int)frameDimension.getHeight();

            // Avoid getting outside the screen.
            if (frameHeight > screenSize.getHeight()) {
                frameDimension.height = ((int)screenSize.getHeight()) - 20;
            }

            mainFrame.setLocation(frameX, frameY);
            mainFrame.setSize(frameDimension);
        }

        mainFrame.setVisible(true);

        // Start the timer to update the progress monitor
        timer = new javax.swing.Timer(QUARTER_OF_SECOND, new TimerListener());
        timer.start();
    }
}
